// OpenGL.cpp : 
//

#include "stdafx.h"
#include "PixFrmt.h"
#include "OpenGL.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

//IMPLEMENT_DYNCREATE(COpenGLView, CView)

COpenGLView::COpenGLView()
{
}

COpenGLView::~COpenGLView()
{
}

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_NCCREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView: 

void COpenGLView::OnDraw(CDC* pDC)
{
	m_pGL->Render();
}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView:  

void COpenGLView::OnInitialUpdate() 
{
  m_pGL->Init();
  CView::OnInitialUpdate();
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
  // OpenGL    
  // WS_CLIPSIBLINGS  WS_CLIPCHILDREN
  cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	
	return CView::PreCreateWindow(cs);
}

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  m_pGL = GetOpenGL();
	ASSERT(m_pGL);
	
  BOOL bResult = m_pGL->Create(this);
	if (bResult)
		return 0;
	else
		return -1;
}

void COpenGLView::OnDestroy() 
{
  m_pGL->Destroy();
  CView::OnDestroy();
}

void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
  m_pGL->Resize(cx,cy);
}

BOOL COpenGLView::OnNcCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (!CView::OnNcCreate(lpCreateStruct))
		return FALSE;
	
  m_pGL = GetOpenGL();
	ASSERT(m_pGL);
		
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// COpenGL

// 
//
COpenGL::COpenGL()
{
	m_hrc = NULL ;
	m_pdc = NULL ;
	
	m_bDoubleBuffer = FALSE ;
}

// 
//
COpenGL::~COpenGL()
{
	Destroy() ;
}

// Destroy: 
//
void COpenGL::Destroy() 
{
	if (m_hrc) 
	{
		//   
		if (m_hrc == wglGetCurrentContext())
			wglMakeCurrent(NULL,NULL) ; 

		wglDeleteContext(m_hrc) ;
		m_hrc = NULL ;
	}		
}

// Create
//
BOOL COpenGL::Create(CWnd* pWnd, int iPixelType, DWORD dwFlags)
{
	ASSERT(pWnd) ;
	ASSERT((iPixelType == PFD_TYPE_RGBA) ||
         (iPixelType == PFD_TYPE_COLORINDEX)) ;

	//   Pixel Format Descriptor
	//
  PIXELFORMATDESCRIPTOR pfd ;
	memset(&pfd,0, sizeof(PIXELFORMATDESCRIPTOR));
  pfd.nSize        = sizeof(PIXELFORMATDESCRIPTOR);  
  pfd.nVersion     = 1;              //  
	pfd.dwFlags      = dwFlags;
	pfd.iPixelType   = iPixelType;
	pfd.cColorBits   = 64;             // 64   
  pfd.cAlphaBits   = 64;             // 64   
  pfd.cAccumBits   = 64;             // 64   
  pfd.cDepthBits   = 64;             // 64-  
  pfd.cStencilBits = 64;             // 64   
  pfd.cStencilBits = 64;             // 64    
  pfd.iLayerType   = PFD_MAIN_PLANE; //  

  //pfd.dwFlags |= (//PFD_DRAW_TO_BITMAP |
  //                PFD_GENERIC_FORMAT |
  //                PFD_NEED_PALETTE   |
  //                PFD_STEREO);//         |
                  //PFD_NEED_SYSTEM_PALETTE);

	//    
  //
	OnCreate(pWnd, &pfd) ;

	//      
  //    Create
	//
	ASSERT( !(dwFlags & PFD_DRAW_TO_BITMAP)) ;
	ASSERT( (dwFlags & PFD_DRAW_TO_WINDOW)) ;

	//    
  //
	m_bDoubleBuffer = pfd.dwFlags & PFD_DOUBLEBUFFER ;

	//   
  //
	m_pdc = new CClientDC(pWnd);

	//     
  //
   int nPixelFormat = ChoosePixelFormat(m_pdc->m_hDC, &pfd);
	if (nPixelFormat == 0)
	{
		TRACE("ChoosePixelFormat Failed %d\r\n",GetLastError()) ;
		return FALSE ;
	}

	//   
  //
   BOOL bResult = SetPixelFormat(m_pdc->m_hDC, nPixelFormat, &pfd);
	if (!bResult)
	{
		TRACE("SetPixelFormat Failed %d\r\n",GetLastError()) ;
		return FALSE ;
	}
	
  //   
  //
  m_hrc = wglCreateContext(m_pdc->m_hDC);
	if (!m_hrc)
	{
		TRACE("wglCreateContext Failed %x\r\n", GetLastError()) ;
		return FALSE;
	}

	//       ,
  //  wglMakeCurrent.      Resize
  //
	return TRUE;
}

//  
//
BOOL COpenGL::Resize(int cx, int cy) 
{
	if ( (cx <= 0) || (cy <= 0) ) return FALSE;

	return OnResize(cx, cy) ;    
}

// 
//
BOOL COpenGL::Init()
{
	return OnInit() ;
}

// 
//
BOOL COpenGL::Render()
{
	// 
	OnRender();

	glFlush();

	//    ,
  //    
  if (m_bDoubleBuffer)
		SwapBuffers(m_pdc->m_hDC);

	return TRUE;
}

